Option Explicit

' Accelerometer interface pin constants and variables
const xAccelInputPin 			as byte = 13
const yAccelInputPin 			as byte = 14
dim xAccel7bit				as byte
dim yAccel7bit				as byte

' Button and pot constants and variables
const redButtonPin 			as byte = 18
const potPin					as byte = 16
dim redButtonState 			as byte
dim previousRedButtonState 		as byte
dim potValue					as integer
dim previousPotValue			as integer

' Create arrays that will function as input and output
' buffers for the serial communication
dim midiOutputBuffer(1 to 20) 	as byte
dim midiInputBuffer(1 to 20) 	as byte
dim lcdOutputBuffer(1 to 60) 	as byte
dim lcdInputBuffer(1 to 10) 	as byte
const lcdOutputPin 			as byte = 5
'dim LCDTaskStack(1 to 101)		as byte

' Define midi constants
' 	Accelerometer data will be transmitted as a noteOn, channel 1, where
' 	the pitch is X acceleration and the velocity is Y acceleration.
' 	The status of the red button will be transmitted as 
'	noteOn, channel 2, note 60. This demonstrates using different
'	midi channels for data.
'	The value of the pot will be transmitted on channel 1 as continuous
'	controller # 1.
const accelStatusByte7bit 		as byte = 144 	' note on, channel 1
const buttonStatusByte 		as byte = 145		' note on, channel 2
const potStatusByte			as byte = 176		' continuous controller, channel 1
const potControllerNumber		as byte = 1
const redButtonNote 			as byte = 60

' Create an array to store one three-byte midi message
' 	Byte 1 = status byte
'	Byte 2 = first data byte
'	Byte 3 = second data byte
dim midiOutput(1 to 3) 		as byte
dim midiInput(1 to 3)			as byte

' For the lcd serial module
dim lcdMessage 				as string
dim lcdControl 				as byte
dim count 					as byte

'---------------------------------------------------------
Public Sub Main()

	putpin 25, 0
	putpin 26, 1
	delay 1.0

	' ** LCD Display module **
	' Define and open COM3 for the LCD display: 2400, 8-n-1
	' 	The LCD display works just like hyperterminal, etc - 
	' 	it will display in ASCII any data sent serially to it.
	'	Some constants are defined in SerialLCD.bas that are special 
	' 	control characters, like "clearScreen" etc. 
	call defineCom3(0, lcdOutputPin, bx1000_1000)
	call openQueue(lcdInputBuffer, 10)
	call openQueue(lcdOutputBuffer, 60)
	call openCom(3, 2400, lcdInputBuffer, lcdOutputBuffer)
	' Send a message to the controller screen.
	lcdMessage = "Initialized" 
	lcdControl = clearScreen
	call putQueue(lcdOutputBuffer, lcdControl, 1)
	call putQueueStr(lcdOutputBuffer, lcdMessage)
	delay 1.0

	' ** MIDI Output **
	' Turn the input, output transfer arrays into FIFO queues
	call openQueue(midiOutputBuffer, 20)
	call openQueue(midiInputBuffer, 20)

	' Open a serial port on the BX's COM1 (pins 1-4)
	' 	The midi-specified baud rate is 31,250;
	' 	the closest the BX can support is 30,270, but
	' 	outbound devices don't seem to mind.
	call openCom(1, 30270, midiInputBuffer, midiOutputBuffer)

	' ** ADXL202eb **
	call callibrateAccel(xAccelInputPin)

	' ** Setup interface variables **
	redButtonState 			= 0
	previousRedButtonState 	= 0
	potValue 					= 0
	previousPotValue 			= 0

	' Turn off onboard LEDs
	putpin 25, 1
	putpin 26, 0

	'call configureMAX3110()

	do

		' Step 1: Read and transmit accelerometer data
		xAccel7bit = readAccel(xAccelInputPin)
		yAccel7bit = readAccel(yAccelInputPin)

		' To transmit the data via midi, we'll use a note on message on channel one.
		' The pitch will reflect the x-axis acceleration, and the velocity the y-axis acceleration.

		' Set status byte
		' 144 = note on, channel 1
		midiOutput(1) = accelStatusByte7bit
		' Set first data byte
		midiOutput(2) = xAccel7bit
		' Set second data byte
		midiOutput(3) = yAccel7bit 
		' Send the data out the midi output buffer
		call putQueue(midiOutputBuffer, midiOutput, 3)


		' Step 2: read and transmit the button data		
		redButtonState = getPin(redButtonPin)

		' Only send data if has changed from the last time
		if redButtonState <> previousRedButtonState then
			midiOutput(1) = buttonStatusByte
			midiOutput(2) = redButtonNote
			midiOutput(3) = redButtonState
			call putQueue(midiOutputBuffer, midiOutput, 3)
			previousRedButtonState = redButtonState
		end if

		' Step 3: read and transmit the button data		
		potValue = getADC(potPin)
		' Only send data if has changed from the last time
		if potValue <> previousPotValue then
			midiOutput(1) = potStatusByte
			midiOutput(2) = potControllerNumber
			' Scale pot data (10 bits) to fit in midi byte (7 bits)
			' To do so, knock off three bits, which means
			' divide by 2^3, which is 8
			midiOutput(3) = cbyte(potValue \ 8)
			call putQueue(midiOutputBuffer, midiOutput, 3)
			previousPotValue = potValue
		end if

		sleep 0.01

	loop

End Sub

